use std::collections::{BTreeSet, HashMap};

use data::tables::{quick_access_template_tb, QuickAccessID, UnlockConfigID};
use proto::{LockModelBin, QuickAccessData, QuickAccessType, UnlockData};

#[derive(Default)]
pub struct LockModel {
    unlock_list: BTreeSet<UnlockConfigID>,
    quick_access_list: HashMap<u32, QuickAccessID>,
}

impl LockModel {
    pub fn from_bin(bin: LockModelBin) -> Self {
        Self {
            unlock_list: bin
                .unlock_list
                .into_iter()
                .map(UnlockConfigID::new_unchecked)
                .collect(),
            quick_access_list: bin
                .quick_access_list
                .into_iter()
                .map(|(k, v)| (k, QuickAccessID::new_unchecked(v)))
                .collect(),
        }
    }

    pub fn to_bin(&self) -> LockModelBin {
        LockModelBin {
            unlock_list: self
                .unlock_list
                .clone()
                .into_iter()
                .map(|i| i.value())
                .collect(),
            quick_access_list: self
                .quick_access_list
                .clone()
                .into_iter()
                .map(|(k, v)| (k, v.value()))
                .collect(),
        }
    }

    pub fn to_client(&self) -> UnlockData {
        let mut quick_access_data_list: Vec<QuickAccessData> = vec![];
        for quick_access in quick_access_template_tb::iter() {
            quick_access_data_list.push(QuickAccessData {
                r#type: QuickAccessType::Direct.into(),
                quick_access_index: quick_access.quick_access_index,
                btn_id: quick_access.quick_func_id.value(),
            });
        }
        for quick_access_index in 1..7 {
            let mut btn_id = 0;
            if let Some(id) = self.quick_access_list.get(&quick_access_index) {
                btn_id = id.value();
            }
            quick_access_data_list.push(QuickAccessData {
                r#type: QuickAccessType::QuickMenu.into(),
                quick_access_index,
                btn_id,
            });
        }

        UnlockData {
            unlock_id_list: self
                .unlock_list
                .clone()
                .into_iter()
                .map(|i| i.value())
                .collect(),
            quick_access_data_list,
            ..Default::default()
        }
    }

    pub fn add_unlock(&mut self, id: UnlockConfigID) {
        self.unlock_list.insert(id);
    }

    pub fn is_unlock(&self, id: UnlockConfigID) -> bool {
        self.unlock_list.contains(&id)
    }

    pub fn mod_quick_access(&mut self, index: u32, id: Option<QuickAccessID>) -> QuickAccessData {
        let btn_id = match id {
            Some(quick_access_id) => {
                if self.quick_access_list.contains_key(&index) {
                    *self.quick_access_list.get_mut(&index).unwrap() = quick_access_id;
                } else {
                    self.quick_access_list.insert(index, quick_access_id);
                }
                quick_access_id.value()
            }
            None => {
                if self.quick_access_list.contains_key(&index) {
                    self.quick_access_list.remove(&index);
                }
                0
            }
        };
        QuickAccessData {
            r#type: QuickAccessType::QuickMenu.into(),
            quick_access_index: index,
            btn_id,
        }
    }
}
